from ready.result import result


# Check: HTTP -> HTTPS redirection occurs
def check_http_to_https_redirect(responses, **kwargs):
    if responses["http_response"]:
        return result(
            responses["http_response"].url.startswith("https://"),
            f"HTTP -> HTTPS redirection ({responses['http_response'].url})",
            "redirect_http",
            **kwargs,
        )

    return result(
        False,
        f"HTTP -> HTTPS redirection (no HTTP response)",
        "redirect_http",
        warn_on_fail=True,
        **kwargs,
    )



import json
import re

from ready import thttp
from ready.result import result


# Check: SPF TXT record should exist
def check_spf_record_should_exist(responses, **kwargs):
    records = [r["data"] for r in responses["dns_txt_response"].json.get("Answer", []) if r["data"].startswith("v=spf")]

    if not records and "dns_txt_response_fld" in responses:
        records = [r["data"] for r in responses["dns_txt_response_fld"].json.get("Answer", []) if r["data"].startswith("v=spf")]

    return result(
        len(records) > 0,
        f"SPF TXT record should exist ({records})",
        "email_spf",
        **kwargs,
    )


# Check: SPF TXT record should contain "-all"
def check_spf_txt_record_should_disallow_all(responses, **kwargs):
    records = [r["data"] for r in responses["dns_txt_response"].json.get("Answer", []) if r["data"].startswith("v=spf")]

    if not records and "dns_txt_response_fld" in responses:
        records = [r["data"] for r in responses["dns_txt_response_fld"].json.get("Answer", []) if r["data"].startswith("v=spf")]

    return result(
        records and all(["-all" in r for r in records]),
        f'SPF TXT record should contain "-all" ({records})',
        "email_spf_disallow_all",
        **kwargs,
    )


# Check: SPF DNS record is depreciated and should not exist
def check_spf_dns_record_does_not_exist(responses, **kwargs):
    records = [r["data"] for r in responses["dns_spf_response"].json.get("Answer", []) if "data" in r]

    if "dns_spf_response_fld" in responses:
        records.extend([r["data"] for r in responses["dns_spf_response_fld"].json.get("Answer", []) if "data" in r])

    return result(
        len(records) == 0,
        f"SPF DNS record is depreciated and should not exist ({records})",
        "email_spf_dns",
        **kwargs,
    )


def _spf_for_domain(domain, depth=0, lookups=[]):
    if domain in lookups:
        return []

    response = thttp.request(f"https://dns.google/resolve?name={domain}&type=TXT")
    lookups.append(domain)

    j = json.loads(response.content)

    spf_records = [(domain, x["data"]) for x in j.get("Answer", []) if x["data"].startswith("v=spf")]

    results = [x for x in spf_records]

    for _, record in spf_records:
        depth += 1
        if depth > 13:
            return results

        matches = re.findall("include\:([^\s]+)", record)

        for d in matches:
            results.extend(_spf_for_domain(d, depth, lookups))

        matches = re.findall("redirect\=([^\s]+)", record)

        for d in matches:
            results.extend(_spf_for_domain(d, depth, lookups))

    return results


# Check: SPF includes use less than 10 DNS requests
def check_spf_uses_less_than_10_requests(responses, **kwargs):
    records = [r["data"] for r in responses["dns_txt_response"].json.get("Answer", []) if r["data"].startswith("v=spf")]

    if not records and "dns_txt_response_fld" in responses:
        records = [
            r["data"]
            for r in responses["dns_txt_response_fld"].json.get("Answer", [])
            if "data" in r and r["data"].startswith("v=spf")
        ]

    additional_lookups = []
    for record in records:
        matches = re.findall("include\:([^\s]+)", record)

        for d in matches:
            additional_lookups.extend(_spf_for_domain(d))

        matches = re.findall("redirect\=([^\s]+)", record)

        for d in matches:
            additional_lookups.extend(_spf_for_domain(d))

    return result(
        len(additional_lookups) <= 10,
        f"SPF includes use less than 10 DNS requests ({len(additional_lookups)})",
        "email_spf_recursion",
        **kwargs,
    )


# Check: DMARC record should exist
def check_dmarc_record_should_exist(responses, **kwargs):
    records = [r["data"] for r in responses["dns_dmarc_response"].json.get("Answer", []) if "data" in r]

    if not records and "dns_dmarc_response_fld" in responses:
        records = [r["data"] for r in responses["dns_dmarc_response_fld"].json.get("Answer", []) if "data" in r]

    return result(
        records and all([r.startswith("v=DMARC1") for r in records]),
        f"DMARC record should exist ({records})",
        "email_dmarc_exists",
        **kwargs,
    )


# Check: DMARC record should contain p=reject
def check_dmarc_record_should_reject_failures(responses, **kwargs):
    records = [r["data"] for r in responses["dns_dmarc_response"].json.get("Answer", []) if "data" in r]

    if not records and "dns_dmarc_response_fld" in responses:
        records = [r["data"] for r in responses["dns_dmarc_response_fld"].json.get("Answer", []) if "data" in r]

    failing = True

    for r in records:
        if "p=reject" in [x.strip().lower() for x in r.split(";")]:
            failing = False

    return result(
        not failing,
        f"DMARC record should contain p=reject ({records})",
        "email_dmarc_none",
        **kwargs,
    )


# Check: SPF should be "v=spf1 -all" if there are no MX records or MX record is "."
def check_spf_dash_all(responses, **kwargs):
    # return none if there is an mx record
    mx_records = []
    if responses["dns_mx_response"] and "Answer" in responses["dns_mx_response"].json:
        mx_records.extend(responses["dns_mx_response"].json["Answer"])

    if "dns_mx_response_fld" in responses and "Answer" in responses["dns_mx_response_fld"].json:
        mx_records.extend(responses["dns_mx_response_fld"].json["Answer"])

    mx_record_data = [r["data"] for r in mx_records]

    if len(mx_record_data) == 0 or all([r == "0 ." for r in mx_record_data]):
        spf_records = [r["data"] for r in responses["dns_txt_response"].json.get("Answer", []) if r["data"].startswith("v=spf")]

        if spf_records:
            spf_record = spf_records[0]
        else:
            spf_record = ""

        return result(
            spf_record.lower().strip() == "v=spf1 -all",
            f"SPF should be 'v=spf1 -all' if there are no MX records or MX record is '.' ({spf_record})",
            "email_spf_disallow_all_with_empty_mx",
            **kwargs,
        )
    else:
        return result(
            True,
            f"SPF should be 'v=spf1 -all' if there are no MX records or MX record is '.' (MX records exist: {mx_record_data})",
            "email_spf_disallow_all_with_empty_mx",
            **kwargs,
        )



from ready.result import result
from ready.thttp import request


# WIP: GraphQL introspection should not be enabled (requires --fuzz)
def check_graphql_introspection_should_not_be_enabled(requests, **kwargs):
    # https://owasp.org/www-project-web-security-testing-guide/latest/4-Web_Application_Security_Testing/12-API_Testing/01-Testing_GraphQL#introspection-queries
    # https://the-bilal-rizwan.medium.com/graphql-common-vulnerabilities-how-to-exploit-them-464f9fdce696
    paths = [
        "graphql"
        "api"
        "api/graphql"
        "api/graphiql"
        "graphiql"
        "v1/graphql"
        "v2/graphql"
        "v3/graphql"
        "v1/graphiql"
        "v2/graphiql"
        "v3/graphiql"
        "console"
        "playground"
        "gql"
        "index.php%3Fgraphql"
        "graphql/"
        "api/"
        "api/graphql/"
        "api/graphiql/"
        "graphiql/"
        "v1/graphql/"
        "v2/graphql/"
        "v3/graphql/"
        "v1/graphiql/"
        "v2/graphiql/"
        "v3/graphiql/"
        "console/"
        "playground/"
        "gql"
    ]



from ready.result import result


# Check: Response should include a Content-Type
def check_http_response_should_include_content_type(responses, **kwargs):
    return result(
        responses["response"].headers.get("content-type") != None,
        f"Response should include a Content-Type ({responses['response'].headers.get('content-type')})",
        "http_content_type",
        **kwargs,
    )


# Check: Response should be gzipped
def check_http_response_should_be_gzipped(responses, **kwargs):
    return result(
        "gzip" in responses["response"].headers.get("content-encoding", ""),
        f"Response should be gzipped ({responses['response'].headers.get('content-encoding', '')})",
        "http_gzipped",
        **kwargs,
    )


# Check: Content-Type header should contain charset
def check_http_content_type_header_contains_charset(responses, **kwargs):
    return result(
        "charset=" in responses["response"].headers.get("content-type", ""),
        f'Content-Type header should contain charset ({responses["response"].headers.get("content-type", "")})',
        "http_charset",
        **kwargs,
    )


# Check: Expires header is deprecated and should not be returned
def check_http_expires_header_is_not_set(responses, **kwargs):
    return result(
        "expires" not in responses["response"].headers,
        f"Expires header is deprecated and should not be returned ({responses['response'].headers.get('expires')})",
        "http_expires",
        **kwargs,
    )


# Check: Cache-Control header should be included in the response
def check_http_cache_control_is_included(responses, **kwargs):
    return result(
        "cache-control" in responses["response"].headers,
        f"Cache-Control header should be included in the response ({responses['response'].headers.get('cache-control')})",
        "http_expires",
        **kwargs,
    )


# Check: P3P header is deprecated and should not be returned
def check_http_p3p_header_is_not_set(responses, **kwargs):
    return result(
        "p3p" not in responses["response"].headers,
        f"P3P header is deprecated and should not be returned ({responses['response'].headers.get('p3p')})",
        "http_p3p",
        **kwargs,
    )



from ready.result import result


# Check: At least two nameservers should be configured
def check_at_least_two_nameservers_configured(responses, **kwargs):
    nameservers = [x["data"] for x in responses["dns_ns_response"].json.get("Answer", [])]

    if not nameservers and "dns_ns_response_fld" in responses:
        nameservers = [x["data"] for x in responses["dns_ns_response_fld"].json.get("Answer", [])]

    return result(
        len(nameservers) > 1,
        f"At least two nameservers should be provided ({nameservers})",
        "ns_minimum_count",
        **kwargs,
    )



from unittest import TestCase

from ready.checks.ns import check_at_least_two_nameservers_configured
from ready.thttp import Response


class NameserversTestCase(TestCase):
    def test_subdomain(self):
        # this should make an additional dns request
        responses = {
            "dns_ns_response": Response(None, None, {"Answer": []}, None, None, None, None),
            "dns_ns_response_fld": Response(
                None,
                None,
                {
                    "Answer": [
                        {"data": "ns1.example.com"},
                        {"data": "ns2.example.com"},
                        {"data": "ns3.example.com"},
                        {"data": "ns3.example.com"},
                    ]
                },
                None,
                None,
                None,
                None,
            ),
        }
        result = check_at_least_two_nameservers_configured(responses, domain="subdomain.example.com", print_output=False)
        self.assertTrue(result.passed)



from unittest import TestCase

from ready.checks.bad_response import (
    check_bad_response_cloudflare,
    check_bad_response_kasada,
)
from ready.thttp import Response


class CloudflareTestCase(TestCase):
    def test_cloudflare_blocked_content(self):
        content = b"""<!DOCTYPE html>\n<html>\n<head>\n  <meta content="width=device-width, initial-scale=1, maximum-scale=1" name="viewport">\n  <title>Checking your Browser - GitLab</title>\n  <style>body{align-items:center;color:#666;display:flex;flex-direction:column;font-family:Helvetica Neue,Helvetica,Arial,sans-serif;font-size:14px;justify-content:center;margin:auto;text-align:center}hr{border:0;border-bottom:1px solid #fff;border-top:1px solid #eee;margin:18px auto;max-width:800px}img{max-width:40vw}.container{margin:auto 20px}.cferror_details{list-style-type:none}.cf-error-details h1{color:#456;font-size:20px;font-weight:400;line-height:28px}</style>\n<meta http-equiv="refresh" content="35">\n</head>\n\n<body>\n  <h1>\n    <img src="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjAwIiBoZWlnaHQ9IjE5MiIgdmlld0JveD0iMCAwIDI1IDI0IiBmaWxsPSJub25lIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPgogIDxwYXRoIGQ9Im0yNC41MDcgOS41LS4wMzQtLjA5TDIxLjA4Mi41NjJhLjg5Ni44OTYgMCAwIDAtMS42OTQuMDkxbC0yLjI5IDcuMDFINy44MjVMNS41MzUuNjUzYS44OTguODk4IDAgMCAwLTEuNjk0LS4wOUwuNDUxIDkuNDExLjQxNiA5LjVhNi4yOTcgNi4yOTcgMCAwIDAgMi4wOSA3LjI3OGwuMDEyLjAxLjAzLjAyMiA1LjE2IDMuODY3IDIuNTYgMS45MzUgMS41NTQgMS4xNzZhMS4wNTEgMS4wNTEgMCAwIDAgMS4yNjggMGwxLjU1NS0xLjE3NiAyLjU2LTEuOTM1IDUuMTk3LTMuODkuMDE0LS4wMUE2LjI5NyA2LjI5NyAwIDAgMCAyNC41MDcgOS41WiIKICAgICAgICBmaWxsPSIjRTI0MzI5Ii8+CiAgPHBhdGggZD0ibTI0LjUwNyA5LjUtLjAzNC0uMDlhMTEuNDQgMTEuNDQgMCAwIDAtNC41NiAyLjA1MWwtNy40NDcgNS42MzIgNC43NDIgMy41ODQgNS4xOTctMy44OS4wMTQtLjAxQTYuMjk3IDYuMjk3IDAgMCAwIDI0LjUwNyA5LjVaIgogICAgICAgIGZpbGw9IiNGQzZEMjYiLz4KICA8cGF0aCBkPSJtNy43MDcgMjAuNjc3IDIuNTYgMS45MzUgMS41NTUgMS4xNzZhMS4wNTEgMS4wNTEgMCAwIDAgMS4yNjggMGwxLjU1NS0xLjE3NiAyLjU2LTEuOTM1LTQuNzQzLTMuNTg0LTQuNzU1IDMuNTg0WiIKICAgICAgICBmaWxsPSIjRkNBMzI2Ii8+CiAgPHBhdGggZD0iTTUuMDEgMTEuNDYxYTExLjQzIDExLjQzIDAgMCAwLTQuNTYtMi4wNUwuNDE2IDkuNWE2LjI5NyA2LjI5NyAwIDAgMCAyLjA5IDcuMjc4bC4wMTIuMDEuMDMuMDIyIDUuMTYgMy44NjcgNC43NDUtMy41ODQtNy40NDQtNS42MzJaIgogICAgICAgIGZpbGw9IiNGQzZEMjYiLz4KPC9zdmc+Cg==" alt="GitLab" /><br />\n  </h1>\n  <div class="container">\n    <div class="cf-browser-verification cf-im-under-attack">\n    <noscript>\n        <h1 style="color:#bd2426;">Please turn JavaScript on and reload the page.</h1>\n    </noscript>\n    <div id="cf-content">\n        <div id="cf-bubbles">\n            <div class="bubbles"></div>\n            <div class="bubbles"></div>\n            <div class="bubbles"></div>\n        </div>\n        <h1>Checking your browser before accessing gitlab.com.</h1>\n        <div id="no-cookie-warning" class="cookie-warning" style="display:none">\n            <p style="color:#bd2426;">Please enable Cookies and reload the page</p>\n        </div>\n        <p>This process is automatic. Your browser will redirect to your requested content shortly.</p>\n        <p id="cf-spinner-allow-5-secs">Please allow up to 5 seconds...</p>\n        <p id="cf-spinner-redirecting" style="display:none">Redirecting...</p>\n    </div>\n    <form id="challenge-form" class="challenge-form" action="/users/sign_in?__cf_chl_f_tk=yEH1.9z1LJpq3YZNo8N4wTDNIXgr.S8eP24Qz64n0Fo-1669515374-0-gaNycGzNB2U" method="POST" enctype="application/x-www-form-urlencoded">\n        <input type="hidden" name="md" value="r7VIAC78sS1BspamhWJ.ptPz5XGfuQ.kf35wuQdr7i0-1669515374-0-AevJOGXiyaqlAjRONHGy6FQUFORCrqkwwEJHQUmXxb16L6FFJSVdgQs7yq4EniKMPJHtE76rbTZBjDGiah6h-7IWKpbdTcEw6Wfq2ejAjI__oq8OvLInxXGV21oeW_blfrJbCbmllx7xVPI_IL1yUpL8nuyQu5EgA8DslvvR_k6PXVCyCfzuzP3VEozYHEpGh5I7ENiv1pmqtTpmnMgWxZ4sBbCQoJeZZEDMJaRfKZ9IB2Hd9VTctpRP3jRPCT867of5jJmAvS5ZBPCzWjIUcF6NebLsMNN8itXXwaPUA5UqiFJJBzL2aWTMU78e7RsxkR3iYaRF-a45qtYcAhNQJUAvbTiR3yxy9UQqAlm50Yhn1MdmQmFHV7edMBLXcNKCxMKkfBcsRY0bRSc7DN8ft5JnPVbNYFZYP0uGLdLu--YAICAtQpgxuikMYJQjulT-m0yzzJbbOzIC7zFZXhT88S-owuEvwdH3g69XV5ywu5vpE6FdcaJR5x0UvJY9UmJMsw_VAPRKNlT-0JXHVmcL30IPulUYAnRPC9eWO6wMjW_cr3meEnnMbBTFdjAYdSzBveWB5PY1C3diYwC1hweEjHUohz5cbQ_XnmhNuby7f7JUK7AulkaVpm9zfzZpIEE2xw" />\n        <input type="hidden" name="r" value="U6Wxc2B5aRslEmqihNUeE.CR2KG8tkLuEK2U4TNEuPo-1669515374-0-AafCr7zxiKFclnfzkL9ytEHYXMoVKREPoDtGgdlMII1M7z78fQ8d8hxqPy3nUOw/Sl1kwGgtAP+yi9SM89ZSSvd5P+as7LuIiELLRXPjn2Z7kxOt6oEBS/C6yIvePNMzsGsXLZKFEDZNv8Vo6sn/N9rsrYFSFQhuOSrf1TSqIsm6D0l0qxs3KcjGupmvWklSUrv01fT8vPYfDQRXlcYf39XFVfhzZn5B+FjuZcULRyHyXc3ralHTvPIOqeua09caMIadSVW6vBd5Za6BUMiHEPQ+ph06V9ZQxZPe7lYrdjQrCRWSVDqV8hla6ORM0KhOolPILY/irUPM8EkWuhx/7H2PKVG9hDlP5w2p1NedQodtBixuTy9LMeVhAVOWIf5ufH8CMBN47HBjVt0PEzuhX0wWnuBVMcMciS0zsibASDkt8dcoxUga7xaFG07Ho5kdNH4Jnp6z0mltRkwtInr9KF/R8f8CPP4T3ss3gYYqfUVGA6qqGzzFflH1wyVvYRXo611hztyYRtXNVsbwnU7FIh0AJvKVRpdkTenAJWDVvJBU0sUAcEwS3svMLYJBVRkOnmoan4bTzr0hBFCfdAvEGD8oAAoB+UftbjJqvSdH2Ydr7lgHgfg1P0fPfSLrY9WoHvm1iPEZ6AEQbKdUI6bNl+4C/ymc1rcZzwNnJUxwHGOOTCepSFsgiAKy3PvsK7rnQ7CzJ5w6DeVx8pZ3LRyqh2hFehDGwm8Z3CTTwh1z61VfefZp7v81eGNc9W51rEt/3ebRP2XK6hu0vuMOiopffFPI4t8n7E3ghIkB5TCKmC3TgQdOEVJsEbkt/U8W1ZNw4rODID5hqSSnCZ5xOms3VkLhhjOTjMj1KXrmOh7ZOAYw0YJsrq36GZt1ZrBvewsi4Y7J0UiCnTHru2TeQO6uWlbUqo6vyg4fxP+0kwQWMhNBzydUsMULhCW6hTHnwYaPrPv8NigEGEDhnJG7s3UJWx6EigC9r4N0AGXzOAH7LaXhKRTuuFyVMtZt8m+lD7NUYzoA7tCEI+nqbnr3NZtdqOFeMhfLXKYm+bGKGVq68DCoGUi+ppgItJkHcdZR42DxXnX3DiOuC6g95s2i2JddNXVGO8AZHGEyJTHCL+UV7wsEOweSUmMmMn0dAOfA+lvt5yZ+/4FZvdH3SxNs2kx5iCbqtm53dpze9s6a0XDH8PxPuQHzHGEU/QXlh6py1EH1VDcTRODKqjtyKOoMOqRgLg5uWx1VUX1HamACPoRKGMCxgS6mg8MqjLau5HiE9rhmpwR/FpFT/ufjhJesYb+61zGc4rqMFCBPEmcivV9VZ6CP15M8CqBqvupXKUhtFxw472x5ttQKuidpzor3kYLXq3WtZngL7FPUZNLe7gco9HLqTMefA2WwX5kR26JR4u7wH8l9i5Y0UBwiwGBRyfnE30u/f+7VsGqG+cm1cXA6AvgQ9tmHsHUq58HKOBwAXJDTiGA8447wopa6W/9rIgbYt2501andjddcMf3aS8cTVYSwoqkv1ajtce4kJvxinkVpDx3Y75aIijG8cbJsbIz4OcSptguA8ZhPp+wyhZFDzUZGI08SL+KQrwnPD56viXOAdSP7/mmANT56YWExSegzMLHno58nkNYcKoyleBdyvIq/SrlOWzOdaPgsq5sceKySlmVlMocO98sq5BSY4ZloUxUxdbXV1LNe2Fjm6TjNcWkgbhKuDCHG/M4gxdoT0xudvED2s7luH78og9CWUgse1pT4y4XZaHNPBsNnfHA2PdFcgs7eXN4islWlU8Vy+0BgKoAE3oWI36juj84Hf7G3j1ywg8tHrgKNGpQcIEUqUv5i" />\n    </form>\n    <div id="trk_jschal_js" style="display:none;background-image:url(\'/cdn-cgi/images/trace/jsch/nojs/transparent.gif?ray=77075c526dfea7e9\')"></div>\n    <script>\n    (function(){\n        window._cf_chl_opt={\n            cvId: \'2\',\n            cType: \'non-interactive\',\n            cNounce: \'47132\',\n            cRay: \'77075c526dfea7e9\',\n            cHash: \'748885c987b07f9\',\n            cUPMDTk: "\\/users\\/sign_in?__cf_chl_tk=yEH1.9z1LJpq3YZNo8N4wTDNIXgr.S8eP24Qz64n0Fo-1669515374-0-gaNycGzNB2U",\n            cFPWv: \'b\',\n            cTTimeMs: \'1000\',\n            cTplV: 1,\n            cTplB: \'cf\',\n            cRq: {\n                ru: \'aHR0cHM6Ly9naXRsYWIuY29tL3VzZXJzL3NpZ25faW4=\',\n                ra: \'TW96aWxsYS81LjAgKE1hY2ludG9zaDsgSW50ZWwgTWFjIE9TIFggMTAuMTU7IHJ2OjEwNS4wKSBHZWNrby8yMDEwMDEwMSBGaXJlZm94LzEwNS4w\',\n                rm: \'R0VU\',\n                d: \'gBJ7mEYgJmH071oiC/h/An74LaWKe7K5bGGUOnYgpqgbicGiWDMt8ZLuhSTDkRA/ibo2IBdG9zk2Oo6pZw9BwlTewTiUVU92UgE4L+02nE7e9bIy9+n15U+yE2GQ5rZUWYPxzjflAIY7bUXy+oRHlKe88qANliPKYyl3iBVigiQf/IQIzMuv/gZH/rcL+81RM+LSVRzMtHjWlGHOODHwo85sfJJlGUK9xSa+3TaJbiCwiByYL0DNHTd99xMz0OPqe2+R0x8loOHUjXna9iXnWRQkXrZWdwtcSWypo2IK8N7Nhg/ErpBXu9rZzs4oP1LmV8yZI4BWJCI9q6G69F9qPVc8AFTZQQFjSiUkJRTjJ/V/uNjJ5EQYhQLr+kFbIuoarrWTp6/Yf3RHH1ASKLbbqyMtY1W/81HH1zLkggVs6JTvSVWWpEVmZOSaLAeKj/ilqeDnZADbGgDoHGObKeLIqavqfLUp8keBaVHwB0YNNnxK7hk0u2Qx1hpkOZeVA0OfXapG9Mc7eUeAigO/6ARxSAXWHUPTxoR9wfloBljTWcOc4YrglzzjRK7LJUGFAiyY8MhxSLMqa2l++9S1kBAFkQ==\',\n                t: \'MTY2OTUxNTM3NC40NjYwMDA=\',\n                m: \'nvVYc8gEJ1urWtHiqexBVXUxMNwsm8iGLgT6iezk6hk=\',\n                i1: \'qtCItyxgZQG7YO4Iyndl6g==\',\n                i2: \'xu6XribgPGqPlhnbw58x2A==\',\n                zh: \'UnJDyo9FrWqFHAM0dnr3Qrw4Ll86sU7CDCiveOM3u04=\',\n                uh: \'uozst7PRgePQRqvQWemQfmDZ4ySMjlf5zzU++wP5zWQ=\',\n                hh: \'lGQbzypQ55vDVgAzrKu7GNtEf1PrcZK3oqezycJ79vw=\',\n            }\n        };\n        var trkjs = document.createElement(\'img\');\n        trkjs.setAttribute(\'src\', \'/cdn-cgi/images/trace/jsch/js/transparent.gif?ray=77075c526dfea7e9\');\n        trkjs.setAttribute(\'style\', \'display: none\');\n        document.body.appendChild(trkjs);\n        var cpo = document.createElement(\'script\');\n        cpo.src = \'/cdn-cgi/challenge-platform/h/b/orchestrate/jsch/v1?ray=77075c526dfea7e9\';\n        window._cf_chl_opt.cOgUHash = location.hash === \'\' && location.href.indexOf(\'#\') !== -1 ? \'#\' : location.hash;\n        window._cf_chl_opt.cOgUQuery = location.search === \'\' && location.href.slice(0, -window._cf_chl_opt.cOgUHash.length).indexOf(\'?\') !== -1 ? \'?\' : location.search;\n        if (window.history && window.history.replaceState) {\n            var ogU = location.pathname + window._cf_chl_opt.cOgUQuery + window._cf_chl_opt.cOgUHash;\n            history.replaceState(null, null, "\\/users\\/sign_in?__cf_chl_rt_tk=yEH1.9z1LJpq3YZNo8N4wTDNIXgr.S8eP24Qz64n0Fo-1669515374-0-gaNycGzNB2U" + window._cf_chl_opt.cOgUHash);\n            cpo.onload = function() {\n                history.replaceState(null, null, ogU);\n            };\n        }\n        document.getElementsByTagName(\'head\')[0].appendChild(cpo);\n    }());\n</script>\n\n</div>\n\n    <hr />\n  </div>\n</body>\n</html>"""

        responses = {"response": Response(None, content, None, 200, None, None, None)}

        result = check_bad_response_cloudflare(responses, print_output=False)
        self.assertFalse(result.passed)


class KasadaTestCase(TestCase):
    def test_kasada_blocked_content(self):
        content = b"""<!DOCTYPE html><html><head></head><body><script>window.KPSDK={};KPSDK.now=typeof performance!=='undefined'&&performance.now?performance.now.bind(performance):Date.now.bind(Date);KPSDK.start=KPSDK.now();</script><script src="/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/ips.js?KP_UIDz=0D9xVG2Ip5eHSs6ZwGDJpAGT6uIWNHUgspsmZEfLljMmX77gXo6bBoMN20oeZuHoeDAkszo9OCe5IVNHjNjyv7ICjEyf1P3ZgWgqgN0ru799vvQbYCKbSEjpWL9efcxFtnVwGcHCoSMwk0zOO6PpJq6FD"></script></body></html>"""

        responses = {"response": Response(None, content, None, 429, None, None, None)}
        result = check_bad_response_kasada(responses, print_output=False)
        self.assertFalse(result.passed)

    def test_kasada_normal_content(self):
        content = b"""<<!doctype html>
        <html lang="en">
        <head>
            <meta charset="utf-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <meta name="description" content="">
            <title>Minimal base.html</title>
        </head>
        <body>

            <!-- Delete this part -->
            <h1>base.html</h1>
            <p>The absolute minimum <code>base.html</code> to get your project started.</p>
            <h3>Usage:</h3>
            <pre>
              curl https://basehtml.xyz &gt; base.html
            </pre>
            <a href="https://github.com/sesh/base.html">more info</a>
            <script src="/149e9513-01fa-4fb0-aad4-566afd725d1b/2d206a39-8ed7-437e-a3be-862e0f06eea3/ips.js?KP_UIDz=0D9xVG2Ip5eHSs6ZwGDJpAGT6uIWNHUgspsmZEfLljMmX77gXo6bBoMN20oeZuHoeDAkszo9OCe5IVNHjNjyv7ICjEyf1P3ZgWgqgN0ru799vvQbYCKbSEjpWL9efcxFtnVwGcHCoSMwk0zOO6PpJq6FD"></script>
        </body>
        </html>"""

        responses = {"response": Response(None, content, None, 200, None, None, None)}
        result = check_bad_response_kasada(responses, print_output=False)
        self.assertTrue(result.passed)



from ready.result import result


# Check: Robots.txt exists and is a text file
def check_robots_txt_exists(responses, **kwargs):
    robots_response = responses["robots_txt_response"]

    return result(
        robots_response and robots_response.status == 200 and "text/plain" in robots_response.headers.get("content-type", ""),
        "Robots.txt exists and is a text file",
        "wellknown_robots",
        **kwargs
    )


# Check: Security.txt exists and is a text file that contains required attributes
def check_security_txt_exists(responses, **kwargs):
    security_txt_response = responses["security_txt_response"]

    return result(
        security_txt_response
        and security_txt_response.status == 200
        and "text/plain" in security_txt_response.headers.get("content-type", "")
        and b"Contact:" in security_txt_response.content
        and b"Expires:" in security_txt_response.content,
        "Security.txt exists and is a text file that contains required attributes",
        "wellknown_security",
        **kwargs
    )


# Check: Favicon is served at /favicon.ico
def check_favicon_is_served(responses, **kwargs):
    favicon_response = responses["favicon_response"]
    return result(
        favicon_response.status == 200
        and favicon_response.headers.get("content-type", "").startswith("image/"),
        "Favicon is served at /favicon.ico",
        "wellknown_favicon",
        **kwargs
    )



from ready.result import result


# Check: Cross-Origin-Resource-Policy should be "same-origin"
def check_cross_origin_resource_policy_should_be_sameorigin(responses, **kwargs):
    return result(
        responses["response"].headers.get("cross-origin-resource-policy", "") == "same-origin",
        f"Cross-Origin-Resource-Policy header should be same-origin ({responses['response'].headers.get('cross-origin-resource-policy')})",
        "http_corp",
        warn_on_fail=True,
        **kwargs,
    )


# Check: cross-origin-opener-policy should be "same-origin"
def check_cross_origin_opener_policy_should_be_sameorigin(responses, **kwargs):
    return result(
        responses["response"].headers.get("cross-origin-opener-policy", "") == "same-origin",
        f"Cross-Origin-Opener-Policy header should be same-origin ({responses['response'].headers.get('cross-origin-opener-policy')})",
        "http_coop",
        warn_on_fail=True,
        **kwargs,
    )


# Check: Cross-Origin-Embedder-Policy should be "require-corp"
def check_cross_origin_embedder_policy_should_be_require_corp(responses, **kwargs):
    return result(
        responses["response"].headers.get("cross-origin-embedder-policy", "") == "require-corp",
        f"Cross-Origin-Embedder-Policy header should be require-corp ({responses['response'].headers.get('cross-origin-embedder-policy')})",
        "http_coep",
        warn_on_fail=True,
        **kwargs,
    )



from ready.result import result


# Check: Response should be a 200 (after redirects)
def check_http_response_should_be_200(responses, **kwargs):
    return result(
        responses["response"] and responses["response"].status == 200,
        f"Response should be a 200 ({getattr(responses['response'], 'status')} - {getattr(responses['response'], 'url')})",
        "https_status",
        **kwargs,
    )



import re

from ready.result import result


# Check: Response should not contain hints of a Cloudflare captcha page
def check_bad_response_cloudflare(responses, **kwargs):
    return result(
        'div id="cf-content"' not in responses["response"].content.decode(errors="ignore"),
        f"Response should not contain hints of a Cloudflare captcha page",
        "bad_cloudflare",
        warn_on_fail=True,
        **kwargs,
    )


# Check: Response should not contain hints of a Kasada error page
def check_bad_response_kasada(responses, **kwargs):
    uuid_pattern = r"[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}"
    failing = False

    if responses["response"].status == 429:
        if re.search(uuid_pattern + r"/" + uuid_pattern, responses["response"].content.decode(errors="ignore")):
            failing = True

    return result(
        not failing,
        f"Response should not contain hints of a Kasada error page",
        "bad_kasada",
        warn_on_fail=True,
        **kwargs,
    )



import re

from ready.result import result


# Check: HSTS Header should be included in response
def check_hsts_header_should_be_included_in_response(responses, **kwargs):
    return result(
        responses["response"].headers.get("strict-transport-security") != None,
        f"HSTS Header should be included in response ({responses['response'].headers.get('strict-transport-security')})",
        "ssl_hsts",
        **kwargs,
    )


# Check: HSTS Header should have a long max-age
def check_hsts_header_should_have_a_long_max_age(responses, **kwargs):
    try:
        hsts = responses["response"].headers.get("strict-transport-security", "")
        max_age_re = re.compile("max-age=(\d+)", re.IGNORECASE)
        m = max_age_re.match(hsts)
        max_age = int(m.groups()[0])
        if max_age < 31536000:
            raise Exception
        return result(
            True,
            f"HSTS Header should have a long max-age ({hsts})",
            "ssl_hsts_duration",
            **kwargs,
        )
    except:
        return result(
            False,
            f"HSTS Header should have a long max-age ({hsts})",
            "ssl_hsts_duration",
            **kwargs,
        )


# Check: HSTS Header should have includeSubdomains
def check_hsts_header_should_have_includesubdomains(responses, **kwargs):
    hsts = responses["response"].headers.get("strict-transport-security", "")
    return result(
        "includesubdomains" in hsts.lower(),
        f"HSTS Header should have includeSubdomains ({hsts})",
        "ssl_hsts_subdomains",
        **kwargs,
    )


# Check: HSTS Header should have preload
def check_hsts_header_should_have_preload(responses, **kwargs):
    hsts = responses["response"].headers.get("strict-transport-security", "")
    return result(
        "preload" in hsts.lower(),
        f"HSTS Header should have preload ({hsts})",
        "ssl_hsts_preload",
        **kwargs,
    )
